/*
 * C0220BiZ_Lib.h
 * Software to accompany the article -
 * "The Quark D2000 Development Board: I2C Interfacing"
 * Header for I2C operation of the Newhaven
 * C022BiZ-FSW-FBW-3VM 2 X 20 COG display
 *****************************************************
 ** This software is offered strictly as is with no **
 ** warranties whatsoever. Use it at your own risk. **
 *****************************************************
 */
 
/* Note: GPIO and I2C must be configured before first use */

#ifndef __C0220BiZ_LIB_H__
#define __C0220BiZ_LIB_H__

/* Globals */
static const uint8_t lcdcmd = 0x00; /* command send         */
static const uint8_t lcddta = 0x40; /* data send            */


/* Defines */
#define LCDADDR (0x3c)   /* I2C address (0x78 shifted)  */
#define LCDRST (5)       /* port bit connected to the LCD reset pin   */

/* Function Prototypes */

/**
 * Hardware reset of the C0220BiZ
 *  assumes GPIO has been configured with GPIO5 (DI4) configured as output
 *  and connected to the C0220BiZ RST pin
 *  NOTE: the 40 ms delay is required after reset and before initialization
 *  NOTE: ST7036 data sheet specifies > 100 us reset low time 
 */
void LCD_reset(void);

/**
 * Initialize the C0220BiZ
 * we follow the ST7036 guidelines with specific delays as recommended
 * in the ST7036 data sheets and by Newhaven
 *
 *	 Command sequence (see data sheets)
 *	 0x00 command specifier
 *	 0x38 function set table 0
 *	 delay 10 ms
 *	 0x00 command specifier
 *	 0x39 function set table 1
 *	 delay 10 ms
 *	 0x14 set bias
 *	 0x73 set contrast (low)
 *	 0x5E set icon display /booster on/ contrast (high)
 *	 0x6d set follower control
 *	 delay 300 ms
 *	 0x00 command specifier
 *	 0x0C set display on
 *	 0x01 clear display
 *	 delay 2 ms
 *	 0x00 command specifier
 *	 0x06 set entry mode
 *	 delay 2 ms
 *
 *	 returns 0 on success or errno on the first failure
 */
int LCD_init(void);

/**
 * Clear the C022BiZ screen and set DDRAM address to 0x00
 * returns 0 on success or errno on failure
*/
int LCD_clr(void);

/**
 * Set DDRAM address to 0x00
 * returns 0 on success or errno on failure
 */
int LCD_home(void);

/**
 * Turn display on or off
 *  arg=0, turn display off
 *  arg=1, turn display on
 *  other values are ignored
 *  returns 0 on success or errno on failure
 *  NOTE: cursor value goes back to default off
 */
int LCD_display(uint8_t arg);

/**
 * Turn cursor and blink on or off
 * arg=0, cursor off
 * arg=1, cursor on, blink off
 * arg=2, cursor on, blink on
 * arg=3, cursor off, blink on
 * other values are ignored
 * returns 0 on success or errno on failure
 */
int LCD_cursor(uint8_t arg);

/**
 * Write the character string (null terminated) at the current DDRAM location
 * returns 0 on success or errno on first failure
 * NOTE: no error checking is done for the length of the string
 * (can go beyond display DDRAM)
 */
int LCD_write(const void * message);

/**
 * Writes the character string (null terminated)
 * at the y (line) and x (position) on display
 * x=0-19 0 (first position) 19 (last position)
 * y=0 or 1 (first or second line)
 * out of range arguments are ignored and return with -999
 * otherwise returns 0 on success or errno on first failure
 */
int LCD_writexy(uint8_t x, uint8_t y, const void * message);

/**
* Write a single byte to DDRAM (current location)
* returns 0 on success or errno on failure
*/
int LCD_writedat(uint8_t datum);

/**
 * Writes a single byte at x (position) and y (line)  
 * x=0 (first position) to 19 (last position)
 * y=0 or 1 (first or second line)
 * out of range arguments are ignored and returns with -999
 * otherwise returns 0 on success or errno on first failure
 */
int LCD_writedatxy(uint8_t x, uint8_t y, uint8_t datum);


/**
 * Set DDRAM address to line and position
 * x=0 (first position) to 19 (last position)
 * y=0 or 1 (first or second line)
 * out of range arguments are ignored and return with -999
 * otherwise returns 0 on success or errno on first failure
 */
int LCD_gotoxy(uint8_t x, uint8_t y);


/**
 * Set contrast (low 4 bits of contrast
 * accepts values of 0 (low) - 15 (high)
 * out of range arguments are ignored and return with -999
 * otherwise returns 0 on success or errno on failure
 * NOTE uses Instruction table 1 which is set in LCD_init()
 * we also explicitly set it
 */
int LCD_contrast(uint8_t arg);


/** Install custom character
 * writes the custom character data to CGRAM
 * addr=CGRAM character address 0-7
 * chrcode= the 8 byte character code array
 * length = number of characters in array (normally 8)
 * Note:  we have to pass the length since an array is only going to
*  pass a pointer to element 0
*  returns -999 on an addr error otherwise
*  returns 0 on success or errno on first failure
*/
int LCD_ICC(uint8_t addr, uint8_t * chrcode, uint8_t length);


#endif /*__C0220BiZ_LIB_H__*/

