/*
 * C0220BiZdemo
 * Software to accompany the article -
 * "The Quark D2000 Development Board: I2C Interfacing"
 * Demonstrate the I2C operation of the Newhaven
 * C022BiZ-FSW-FBW-3VM 2 X 20 COG display
 *****************************************************
 ** This software is offered strictly as is with no **
 ** warranties whatsoever. Use it at your own risk. **
 *****************************************************
 */

#include "qm_common.h"
#include "qm_i2c.h"
#include "qm_pinmux.h"
#include "clk.h"
#include <qm_gpio.h>

#define LCDADDR (0x3c)   /* I2C address (0x78 shifted)  */


static const uint8_t lcdcmd = 0x00; /* command send      */
static const uint8_t lcddta = 0x40; /* data send         */

/* GPIO configuration structure (see qm_gpio.h for details)
 After we set the configuration that we want, we can call the qm_gpio_set_config function */
static qm_gpio_port_config_t portcfg;
/*  I2C structure variables  */
static qm_i2c_config_t I2Ccfg;
static qm_i2c_status_t I2Cstatus; /* see i2c.h for values      */

/*___________________________________________________________________________*/

int initI2C() {
	/*  Enable I2C 0 */
	clk_periph_enable(CLK_PERIPH_CLK | CLK_PERIPH_I2C_M0_REGISTER);
	/* set IO pins for SDA and SCL */
	qm_pmux_select(QM_PIN_ID_6, QM_PMUX_FN_2);
	qm_pmux_select(QM_PIN_ID_7, QM_PMUX_FN_2);

	/* Configure I2C */
	I2Ccfg.address_mode = QM_I2C_7_BIT;
	I2Ccfg.mode = QM_I2C_MASTER;
	I2Ccfg.speed = QM_I2C_SPEED_STD;
	/* note at 3.3v the LCD is limited to 300k I2C so set it to standard speed (100 khz) */
	/* set the configuration through the structure and return 0 if failure */
	if (qm_i2c_set_config(QM_I2C_0, &I2Ccfg)) {
		return (errno);
	} else {
		return (0);
	}
}
/*
 void LCD_reset(void)
 * hardware reset of the C0220BiZ
 *  - assumes GPIO has been configured
 *  - RST pin is on GPIO5 (DI4) configured as output
 * NOTE: a 40 ms delay is required after reset and before initialization
 * NOTE: ST7036 data sheet specifies > 100 us reset low time (we use 200us)
 */
void LCD_reset(void) {

	qm_gpio_clear_pin(QM_GPIO_0, QM_PIN_ID_5);
	clk_sys_udelay(200); /* 200 microseconds delay */
	qm_gpio_set_pin(QM_GPIO_0, QM_PIN_ID_5);
}

/*
 int LCD_init(void)
 * initialize the C0220BiZ
 * - we follow the ST7036 guidelines with specific delays as recommended
 * -  in the ST7036 data sheets and by Newhaven
 *
 *	 Command sequence (see data sheets)
 *	 0x00 command specifier
 *	 0x38 function set table 0
 *	 delay 10 ms
 *	 0x00 command specifier
 *	 0x39 function set table 1
 *	 delay 10 ms
 *	 0x14 set bias
 *	 0x73 set contrast (low)
 *	 0x5E set icon display /booster on/ contrast (high)
 *	 0x6d set follower control
 *	 delay 200 ms
 *	 0x00 command specifier
 *	 0x0C set display on
 *	 0x01 clear display
 *	 delay 2 ms
 *	 0x00 command specifier
 *	 0x06 set entry mode
 *	 delay 2 ms
 *
 *	 returns 0 on success or errno on the first failure
 */
int LCD_init() {
	uint8_t c0ini0[2] = { lcdcmd, 0x38 };
	uint8_t c0ini1[2] = { lcdcmd, 0x39 };
	uint8_t c0ini2[5] = { lcdcmd, 0x14, 0x70, 0x5e, 0x6d };
	uint8_t c0ini3[3] = { lcdcmd, 0x0c, 0x01 };
	uint8_t c0ini4[2] = { lcdcmd, 0x06 };
	/* delay for 40 ms upon power up or reset - assume that situation */
	clk_sys_udelay(40000);
	/* note lengths are hard coded */
	if (qm_i2c_master_write(QM_I2C_0, LCDADDR, c0ini0, sizeof(c0ini0), true,
			&I2Cstatus)) {
		return (errno);
	}
	clk_sys_udelay(10000); /* 10 ms after function set table  */
	if (qm_i2c_master_write(QM_I2C_0, LCDADDR, c0ini1, sizeof(c0ini1), true,
			&I2Cstatus)) {
		return (errno);
	}
	clk_sys_udelay(10000); /* 10 ms after function set table  */
	if (qm_i2c_master_write(QM_I2C_0, LCDADDR, c0ini2, sizeof(c0ini2), true,
			&I2Cstatus)) {
		return (errno);
	}
	clk_sys_udelay(200000); /* 200 ms after follower control cmd delay  */
	if (qm_i2c_master_write(QM_I2C_0, LCDADDR, c0ini3, sizeof(c0ini3), true,
			&I2Cstatus)) {
		return (errno);
	}
	clk_sys_udelay(2000); /* 2 ms delay after clear screen */
	if (qm_i2c_master_write(QM_I2C_0, LCDADDR, c0ini4, sizeof(c0ini4), true,
			&I2Cstatus)) {
		return (errno);
	}
	clk_sys_udelay(2000); /* 2 ms delay  */
	return (0);

}
/*
 int LCD_clr(void)
 * clear the C022BiZ screen
 *	 returns 0 on success or errno on failure*
 */
int LCD_clr(void) {
	uint8_t c0clrlcd[2] = { lcdcmd, 0x01 };
	if (qm_i2c_master_write(QM_I2C_0, LCDADDR, c0clrlcd, sizeof(c0clrlcd), true,
			&I2Cstatus)) {
		return (errno);
	}
	clk_sys_udelay(2000);
	return (0);
}

uint8_t writetxt() {
	uint8_t c0txt[1] = { lcddta };
	uint8_t c1txt[2] = { lcdcmd, 0xc0 };
	uint8_t txtline0[] = { "  ALL ABOUT CIRCUITS" };
	uint8_t txtline1[] = { "    Hello World!" };

	/* note false option in next line */
	if (qm_i2c_master_write(QM_I2C_0, LCDADDR, c0txt, sizeof(c0txt), false,
			&I2Cstatus)) {
		return (errno);
	}
	if (qm_i2c_master_write(QM_I2C_0, LCDADDR, txtline0, sizeof(txtline0), true,
			&I2Cstatus)) {
		return (errno);
	}
	/* set RAM to line 2 */
	if (qm_i2c_master_write(QM_I2C_0, LCDADDR, c1txt, sizeof(c1txt), true,
			&I2Cstatus)) {
		return (errno);
	}
	/* note false option in next line */
	if (qm_i2c_master_write(QM_I2C_0, LCDADDR, c0txt, sizeof(c0txt), false,
			&I2Cstatus)) {
		return (errno);
	}
	if (qm_i2c_master_write(QM_I2C_0, LCDADDR, txtline1, sizeof(txtline1), true,
			&I2Cstatus)) {
		return (errno);
	}
	return (0);
}

int lcdwriteset() {
	/*
	 * CGROM has 0x10-0xff for a total of 240 characters
	 */
	uint8_t dta[1] = { lcddta };
	uint8_t cmdLine1[2] = { lcdcmd, 0x80 }; /* line 1 */
	uint8_t cmdLine2[2] = { lcdcmd, 0xc0 }; /* line 2 */
	uint8_t setl1[20];
	uint8_t setl2[20];
	uint8_t x;
	int z;

	for (z = 0; z < 240; z += 40) {
		/* fill the two arrays with 20 bytes each */
		for (x = 0; x < 20; x++) {
			setl1[x] = (uint8_t) (x + z + 16);
			setl2[x] = (uint8_t) (x + z + 16 + 20);
		}
		/* set RAM to line 1 */
		if (qm_i2c_master_write(QM_I2C_0, LCDADDR, cmdLine1, sizeof(cmdLine1),
				true, &I2Cstatus)) {
			return (errno);
		}
		/* write line 1 */
		if (qm_i2c_master_write(QM_I2C_0, LCDADDR, dta, sizeof(dta), false,
				&I2Cstatus)) {
			return (errno);
		}
		if (qm_i2c_master_write(QM_I2C_0, LCDADDR, setl1, sizeof(setl1), true,
				&I2Cstatus)) {
			return (errno);
		}
		/* set RAM to line 2 */
		if (qm_i2c_master_write(QM_I2C_0, LCDADDR, cmdLine2, sizeof(cmdLine2),
				true, &I2Cstatus)) {
			return (errno);
		}
		/* note false option in next line */
		if (qm_i2c_master_write(QM_I2C_0, LCDADDR, dta, sizeof(dta), false,
				&I2Cstatus)) {
			return (errno);
		}
		if (qm_i2c_master_write(QM_I2C_0, LCDADDR, setl2, sizeof(setl2), true,
				&I2Cstatus)) {
			return (errno);
		}
		clk_sys_udelay(3000000); /* 3 s delay  */
	}
	return (0);
}

int main() {

	int error_return;

	QM_PUTS("\n\n\n*** LCD TEST\n");
	/* set GPIO to output by setting the associated bit in the DDR */
	portcfg.direction = 1 <<QM_PIN_ID_5; /* equivalent to portcfg.direction = BIT(LED);*/
	QM_PUTS("Configuring GPIO\n");
	qm_gpio_set_config(QM_GPIO_0, &portcfg);
	qm_gpio_set_pin(QM_GPIO_0, QM_PIN_ID_5); /* set LCDRST high (reset is active low) */
	LCD_reset();
	QM_PUTS("Configuring I2C\n");
	/* configure system I2C */
	if ((error_return = initI2C())) {
		QM_PRINTF("Error: Could not configure I2C - errno:%d\n", errno);
		return 1;
	}
	QM_PUTS("Configuring LCD\n");
	/* configure the C0220BiZ LCD */
	if ((error_return = LCD_init())) {
		QM_PUTS("Error: Could not initialize C0220BiZ LCD\n");QM_PRINTF("errno:%d\n", errno);
		return 1;
	}
	/* loop the screen writing */
	while (1) {
		QM_PUTS("Writing hello message to LCD\n");
		if ((error_return = writetxt())) {
			QM_PRINTF("Error: Could not write hello message to LCD - errno:%d\n", errno);
			return 1;
		}
		clk_sys_udelay(3000000UL); /* 3 s delay  */
		QM_PUTS("Clearing LCD\n");
		if ((error_return = LCD_clr())) {
			QM_PRINTF("Error: could not clear LCD screen - errno:%d\n", errno);
			return 1;
		}
		QM_PUTS("Writing character set to LCD\n");
		if ((error_return = lcdwriteset())) {
			QM_PUTS("Error: Could not write character set to LCD\n");QM_PRINTF("Error: Could not write character set to LCD - errno:%d\n", errno);
			return 1;
		}
		clk_sys_udelay(2000000UL); /* 2 s delay  */
		QM_PUTS("Clearing LCD\n");
		if ((error_return = LCD_clr())) {
			QM_PRINTF("Error: could not clear LCD screen - errno:%d\n", errno);
			return 1;
		}
	}
}
